----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date: 11/27/2017 03:10:09 PM
-- Design Name: 
-- Module Name: SerialControl - Behavioral
-- Project Name: 
-- Target Devices: 
-- Tool Versions: 
-- Description: 
-- 
-- Dependencies: 
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Sources:
--     UART_TX, UART_RX: https://www.nandland.com/vhdl/modules/module-uart-serial-port-rs232.html
--     Debounce: https://eewiki.net/pages/viewpage.action?pageId=4980758#DebounceLogicCircuit(withVHDLexample)-ExampleVHDLCode
--     Division for two unsigned numbers: http://vhdlguru.blogspot.com/2010/03/vhdl-function-for-division-two-signed.html
----------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
--use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx leaf cells in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity SerialControl is
    Generic (
        max_input : STD_LOGIC_VECTOR := "0001111";
        C_B : STD_LOGIC_VECTOR := x"42";
        C_E : STD_LOGIC_VECTOR := x"45";
        C_F : STD_LOGIC_VECTOR := x"46";
        C_I : STD_LOGIC_VECTOR := x"49";
        C_M : STD_LOGIC_VECTOR := x"4D";
        C_S : STD_LOGIC_VECTOR := x"53";
        C_W : STD_LOGIC_VECTOR := x"57";
        a : STD_LOGIC_VECTOR := x"61";
        c : STD_LOGIC_VECTOR := x"63";
        d : STD_LOGIC_VECTOR := x"64";
        e : STD_LOGIC_VECTOR := x"65";
        h : STD_LOGIC_VECTOR := x"68";
        i : STD_LOGIC_VECTOR := x"69";
        l : STD_LOGIC_VECTOR := x"6C";
        m : STD_LOGIC_VECTOR := x"6D";
        n : STD_LOGIC_VECTOR := x"6E";
        o : STD_LOGIC_VECTOR := x"6F";
        p : STD_LOGIC_VECTOR := x"70";
        r : STD_LOGIC_VECTOR := x"72";
        s : STD_LOGIC_VECTOR := x"73";
        t : STD_LOGIC_VECTOR := x"74";
        u : STD_LOGIC_VECTOR := x"75";
        v : STD_LOGIC_VECTOR := x"76";
        w : STD_LOGIC_VECTOR := X"77";
        x : STD_LOGIC_VECTOR := x"78";
        y : STD_LOGIC_VECTOR := x"79";
        zero : STD_LOGIC_VECTOR := x"30";
        one : STD_LOGIC_VECTOR := X"31";
        two : STD_LOGIC_VECTOR := x"32";
        three : STD_LOGIC_VECTOR := x"33";
        l_s_b : STD_LOGIC_VECTOR := x"5B";
        r_s_b : STD_LOGIC_VECTOR := x"5D";
        BS : STD_LOGIC_VECTOR := x"08";
        LF : STD_LOGIC_VECTOR := x"0A";
        CR : STD_LOGIC_VECTOR := x"0D";
        minus : STD_LOGIC_VECTOR := x"2D";
        colon : STD_LOGIC_VECTOR := x"3A";
        tab : STD_LOGIC_VECTOR := x"09";
        space : STD_LOGIC_VECTOR := x"20"
    );
    Port ( RXD : in STD_LOGIC;
--           RXD_DEBUG : out STD_LOGIC;
           TXD : out STD_LOGIC;
--           TXD_DEBUG : out STD_LOGIC;
           CLK : in STD_LOGIC;
--           btnC : in STD_LOGIC;
           sw : in STD_LOGIC_VECTOR (15 downto 0);
           led : out STD_LOGIC_VECTOR (15 downto 0));
--           module_start_debug : out STD_LOGIC;
--           state_debug : out STD_LOGIC_VECTOR(3 downto 0);
--           tx_done_debug : out STD_LOGIC);
end SerialControl;

architecture Behavioral of SerialControl is

function valid (ascii_byte : in std_logic_vector(7 downto 0))
    return boolean is
    begin
        if(ascii_byte = x"30" or ascii_byte = x"31" or ascii_byte = x"32" or ascii_byte = x"33" or ascii_byte = x"34" or ascii_byte = x"35" or ascii_byte = x"36" or ascii_byte = x"37" or ascii_byte = x"38" or ascii_byte = x"39") then
            return true;
        else
            return false;
        end if;
end function;

function ascii_t_bcd (ascii_byte : in std_logic_vector(7 downto 0) )
    return std_logic_vector is
    variable bcd : std_logic_vector(3 downto 0);
    begin
        if(ascii_byte = x"30") then
            bcd := x"0";
        elsif(ascii_byte = x"31") then
            bcd := x"1";
        elsif(ascii_byte = x"32") then
            bcd := x"2";
        elsif(ascii_byte = x"33") then
            bcd := x"3";
        elsif(ascii_byte = x"34") then
            bcd := x"4";
        elsif(ascii_byte = x"35") then
            bcd := x"5";
        elsif(ascii_byte = x"36") then
            bcd := x"6";
        elsif(ascii_byte = x"37") then
            bcd := x"7";
        elsif(ascii_byte = x"38") then
            bcd := x"8";
        elsif(ascii_byte = x"39") then
            bcd := x"9";
        else
            bcd := x"0";
        end if;
    return bcd;
end function;

function bcd_t_binary ( bcd : in std_logic_vector(7 downto 0))
    return std_logic_vector is
    variable binary : std_logic_vector(7 downto 0);
    begin
        binary := (bcd(7 downto 4) * "1010") + (bcd(3 downto 0) * "0001");
    return binary;
end function;

function tens ( value : integer)
    return std_logic_vector is
    variable ascii : std_logic_vector(7 downto 0);
    begin
        if(value < 10) then
            ascii := x"30";
        elsif(value < 20) then
            ascii := x"31";
        elsif(value < 30) then
            ascii := x"32";
        elsif(value < 40) then
            ascii := x"33";
        elsif(value < 50) then
            ascii := x"34";
        elsif(value < 60) then
            ascii := x"35";
        elsif(value < 70) then
            ascii := x"36";
        elsif(value < 80) then
            ascii := x"37";
        elsif(value < 90) then
            ascii := x"38";
        elsif(value < 100) then
            ascii := x"39";
        else
            ascii := x"30";
        end if;
    return ascii;
end function;

function ones (value : integer)
    return std_logic_vector is
    variable ascii : std_logic_vector(7 downto 0);
    variable remainder : integer;
    begin
        remainder := value mod 10;
        if(remainder = 0) then
            ascii := x"30";
        elsif(remainder = 1) then
            ascii := x"31";
        elsif(remainder = 2) then
            ascii := x"32";
        elsif(remainder = 3) then
            ascii := x"33";
        elsif(remainder = 4) then
            ascii := x"34";
        elsif(remainder = 5) then
            ascii := x"35";
        elsif(remainder = 6) then
            ascii := x"36";
        elsif(remainder = 7) then
            ascii := x"37";
        elsif(remainder = 8) then
            ascii := x"38";
        elsif(remainder = 9) then
            ascii := x"39";
        else
            ascii := x"30";
        end if;
    return ascii;
end function;

component UART_TX is
Port (
    i_Clk       : in  std_logic;
    i_TX_DV     : in  std_logic;
    i_TX_Byte   : in  std_logic_vector(7 downto 0);
    o_TX_Active : out std_logic;
    o_TX_Serial : out std_logic;
    o_TX_Done   : out std_logic
    );
end component;

component UART_RX is 
  port (
  i_Clk       : in  std_logic;
  i_RX_Serial : in  std_logic;
  o_RX_DV     : out std_logic;
  o_RX_Byte   : out std_logic_vector(7 downto 0)
  );
 end component;
 
component T_FLIP_FLOP is
Port ( T : in STD_LOGIC;
        CLK : in STD_LOGIC;
        EN : in STD_LOGIC;
        Q : out STD_LOGIC
);
end component;

component debounce is
Port(
    clk     : IN  STD_LOGIC;  --input clock
    button  : IN  STD_LOGIC;  --input signal to be debounced
    result  : OUT STD_LOGIC); --debounced signal
end component;

component solver_complete is
Port ( A : in STD_LOGIC_VECTOR (3 downto 0);
           B : in STD_LOGIC_VECTOR (3 downto 0);
           C : in STD_LOGIC_VECTOR (3 downto 0);
           D : in STD_LOGIC_VECTOR (3 downto 0);
           X : in STD_LOGIC_VECTOR (7 downto 0);
           Y : in STD_LOGIC_VECTOR (7 downto 0);
           U : out STD_LOGIC_VECTOR (7 downto 0);
           V : out STD_LOGIC_VECTOR (7 downto 0)
);
end component;

signal data_tx : STD_LOGIC_VECTOR(7 downto 0);
signal data_rx : STD_LOGIC_VECTOR(7 downto 0);
signal TXD_SIG, send, tx_done, tx_active, received : STD_LOGIC;

signal module_start : STD_LOGIC;
signal state : STD_LOGIC_VECTOR(11 downto 0) := x"000";

signal tab_space : integer := 0;
signal row_bs_count : integer := 0;

signal row1_bcd, row2_bcd : STD_LOGIC_VECTOR(23 downto 0);
signal row1_state, row2_state : STD_LOGIC_VECTOR(5 downto 0) := "000000";
signal r1c1, r1c2, r1c3, r2c1, r2c2, r2c3 : STD_LOGIC_VECTOR(7 downto 0);

signal sol_x, sol_y : STD_LOGIC_VECTOR(7 downto 0);
signal sol_x_int, sol_y_int : integer := 0;

begin


UART1 : UART_TX
Port Map(
    i_Clk => CLK,
    i_TX_DV => send,
    i_TX_Byte => data_tx,
    o_TX_Serial => TXD_SIG,
    o_TX_Active => tx_active,
    o_TX_Done => tx_done
);

UART2 : UART_RX
Port Map(
    i_Clk => CLK,
    i_RX_Serial => RXD,
    o_RX_DV => received,
    o_RX_Byte => data_rx
);

db1 : debounce
Port Map(
    clk => CLK,  --input clock
    button => sw(15),  --input signal to be debounced
    result => module_start --debounced signal
);

solver1 : solver_complete
Port Map(
    A => r1c1(3 downto 0),
    B => r1c2(3 downto 0),
    X => r1c3,
    C => r2c1(3 downto 0),
    D => r2c2(3 downto 0),
    Y => r2c3,
    U => sol_x,
    V => sol_y
);

receive_led : process(received)
begin
    if(rising_edge(received)) then
        led(7 downto 0) <= data_rx;
    end if;
end process;

proc_start_state : process(CLK, module_start)
begin
    if(falling_edge(CLK)) then
        if(module_start = '0' ) then
            state <= x"000"; --RESET state, send DISABLE
            row1_bcd <= x"000000";
            row2_bcd <= x"000000";
            row1_state <= "000000";
            row2_state <= "000000";
        elsif(state = x"000" AND module_start = '1') then --Transition State
            row1_bcd <= x"000000";
            row2_bcd <= x"000000";
            row1_state <= "000000";
            row2_state <= "000000";
            state <= x"001";  --W
        elsif(state = x"001") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= x"002"; --e
            end if;
        elsif(state = x"002") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= x"003"; --l
            end if;
        elsif(state = x"003") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= x"004"; --c
            end if;
        elsif(state = x"004") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= x"005"; --o
            end if;
        elsif(state = x"005") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= x"006"; --m
            end if;
        elsif(state = x"006") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= x"007"; --e
            end if;
        elsif(state = x"007") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= x"008"; --space
            end if;
        elsif(state = x"008") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= x"009"; --t
            end if;
        elsif(state = x"009") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= x"00A"; --o
            end if;
        elsif(state = x"00A") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= x"00B"; --space
            end if;
        elsif(state = x"00B") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= x"00C"; --t
            end if;
        elsif(state = x"00C") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= x"00D"; --h
            end if;
        elsif(state = x"00D") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= x"00E"; --e
            end if;
        elsif(state = x"00E") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= x"00F"; --space
            end if;
        elsif(state = x"00F") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= x"010"; --B
            end if;
        elsif(state = x"010") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= x"011"; --a
            end if;
        elsif(state = x"011") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= x"012"; --s
            end if;
        elsif(state = x"012") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= x"013"; --y
            end if;
        elsif(state = x"013") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= x"014"; --s
            end if;
        elsif(state = x"014") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= x"015"; --three
            end if;
        elsif(state = x"015") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= x"016"; --space
            end if;
        elsif(state = x"016") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= x"017"; --M
            end if;
        elsif(state = x"017") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= x"018"; --a
            end if;
        elsif(state = x"018") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= x"019"; --t
            end if;
        elsif(state = x"019") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= x"01A"; --r
            end if;
        elsif(state = x"01A") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"01B"; --i
            end if;
        elsif(state = x"01B") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"01C"; --x
            end if;
        elsif(state = x"01C") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"01D"; --space
            end if;
        elsif(state = x"01D") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"01E"; --S
            end if;
        elsif(state = x"01E") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"01F"; --o
            end if;
        elsif(state = x"01F") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"020"; --l
            end if;
        elsif(state = x"020") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"021"; --v
            end if;
        elsif(state = x"021") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"022"; --e
            end if;
        elsif(state = x"022") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"023"; --r
            end if;
        elsif(state = x"023") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"024"; --LF
            end if;
        elsif(state = x"024") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"025"; --CR
            end if;
        elsif(state = x"025") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"026"; --E
            end if;
        elsif(state = x"026") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"027"; --n
            end if;
        elsif(state = x"027") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= x"028"; --t
            end if;
        elsif(state = x"028") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= x"029"; --e
            end if;
        elsif(state = x"029") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= x"02A"; --r
            end if;
        elsif(state = x"02A") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"02B"; --space
            end if;
        elsif(state = x"02B") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"02C"; --a
            end if;
        elsif(state = x"02C") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"02D"; --space
            end if;
        elsif(state = x"02D") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"02E"; --two
            end if;
        elsif(state = x"02E") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"02F"; --x
            end if;
        elsif(state = x"02F") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"030"; --three
            end if;
        elsif(state = x"030") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"031"; --space
            end if;
        elsif(state = x"031") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"032"; --m
            end if;
        elsif(state = x"032") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"033"; --a
            end if;
        elsif(state = x"033") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"034"; --t
            end if;
        elsif(state = x"034") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"035"; --r
            end if;
        elsif(state = x"035") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"036"; --i
            end if;
        elsif(state = x"036") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"037"; --x
            end if;
        elsif(state = x"037") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"038"; --colon
            end if;
        elsif(state = x"038") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= x"039"; --LF
            end if;
        elsif(state = x"039") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"03A"; --CR
            end if;
        elsif(state = x"03A") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"03B"; --tab_space
            end if;
        elsif(state = x"03B") then
            if(tx_done = '1' AND tx_active = '0' AND tab_space < 3) then
                tab_space <= tab_space + 1;
            elsif(tx_done = '1' AND tx_active = '0') then
                state <= X"03C"; --F
                tab_space <= 0;
            end if;
        elsif(state = x"03C") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"03D"; --i
            end if;
        elsif(state = x"03D") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"03E"; --r
            end if;
        elsif(state = x"03E") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"03F"; --s
            end if;
        elsif(state = x"03F") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"040"; --t
            end if;
        elsif(state = x"040") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"041"; --space
            end if;
        elsif(state = x"041") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"042"; --r
            end if;
        elsif(state = x"042") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"043"; --o
            end if;
        elsif(state = x"043") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"044"; --w
            end if;
        elsif(state = x"044") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"045"; --colon
            end if;
        elsif(state = x"045") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"046"; -- space
            end if;
        elsif(state = x"046") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"047"; --[
            end if;
        elsif(state = x"047") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"048"; --zero
            end if;
        elsif(state = x"048") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= x"049"; --zero
            end if;
        elsif(state = x"049") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"04A"; --space
            end if;
        elsif(state = x"04A") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"04B"; --zero
            end if;
        elsif(state = x"04B") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"04C"; --zero
            end if;
        elsif(state = x"04C") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"04D"; --space
            end if;
        elsif(state = x"04D") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"04E"; --zero
            end if;
        elsif(state = x"04E") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"04F"; --zero
            end if;
        elsif(state = x"04F") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"050"; --]
            end if;
        elsif(state = x"050") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= x"051"; --BS
            end if;
        elsif(state = x"051") then
            if(tx_done = '1' AND tx_active = '0' AND row_bs_count < 8) then
                row_bs_count <= row_bs_count + 1; --
            elsif(tx_done = '1' AND tx_active = '0') then
                state <= x"052"; --Receive
                row_bs_count <= 0;
            end if;
        elsif(state = x"052") then
            if(received = '1' AND valid(data_rx) ) then
                state <= x"053"; --Send received data
                    if(row1_state(5) = '0') then
                        row1_bcd(23 downto 20) <= ascii_t_bcd(data_rx); --Store input
                        row1_state(5) <= '1';
                    else
                        row1_bcd(19 downto 16) <= ascii_t_bcd(data_rx); --Store input
                        row1_state(4) <= '1';
                    end if;
            end if;
        elsif(state = x"053") then
            if(tx_done = '1' AND tx_active = '0') then
                if(row1_state(4) = '1') then
                    state <= X"054"; --Space
                else
                    state <= X"052"; --Receive
                end if;
            end if;
        elsif(state = x"054") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"055"; --Receive
            end if;
        elsif(state = x"055") then
            if(received = '1' AND valid(data_rx) ) then
                state <= x"056"; --Send received data
                if(row1_state(3) = '0') then
                    row1_bcd(15 downto 12) <= ascii_t_bcd(data_rx); --Store input
                    row1_state(3) <= '1';
                else
                    row1_bcd(11 downto 8) <= ascii_t_bcd(data_rx); --Store input
                    row1_state(2) <= '1';
                end if;
            end if;
        elsif(state = x"056") then
            if(tx_done = '1' AND tx_active = '0') then
                if(row1_state(2) = '1') then
                    state <= X"057"; --Space
                else
                    state <= X"055"; --Receive
                end if;
            end if;
        elsif(state = x"057") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"058"; --Receive
            end if;
        elsif(state = x"058") then
            if(received = '1' AND valid(data_rx) ) then
                state <= x"059"; --Send received data
                if(row1_state(1) = '0') then
                    row1_bcd(7 downto 4) <= ascii_t_bcd(data_rx); --Store input
                    row1_state(1) <= '1';
                else
                    row1_bcd(3 downto 0) <= ascii_t_bcd(data_rx); --Store input
                    row1_state(0) <= '1';
                end if;
            end if;
        elsif(state = x"059") then
            if(tx_done = '1' AND tx_active = '0') then
                if(row1_state(0) = '1') then
                    state <= X"05A"; --LF
                else
                    state <= X"058"; --Receive
                end if;
            end if;
        elsif(state = x"05A") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"05B"; --CR
            end if;
        elsif(state = x"05B") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"05C"; --Space tab
            end if;
        elsif(state = x"05C") then
            if(tx_done = '1' AND tx_active = '0' AND tab_space < 3) then
                tab_space <= tab_space + 1;
            elsif(tx_done = '1' AND tx_active = '0') then
                state <= X"05D"; --S
                tab_space <= 0;
            end if;
        elsif(state = x"05D") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"05E"; --e
            end if;
        elsif(state = x"05E") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"05F"; --c
            end if;
        elsif(state = x"05F") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"060"; --o
            end if;
        elsif(state = x"060") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"061"; --n
            end if;
        elsif(state = x"061") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"062"; --d
            end if;
        elsif(state = X"062") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"063"; --space
            end if;
        elsif(state = x"063") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"064"; --r
            end if;
        elsif(state = x"064") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"065"; --o
            end if;
        elsif(state = x"065") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"066"; --w
            end if;
        elsif(state = x"066") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"067"; --colon
            end if;
        elsif(state = x"067") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"068"; -- space
            end if;
        elsif(state = x"068") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"069"; --[
            end if;
        elsif(state = x"069") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"06A"; --zero
            end if;
        elsif(state = x"06A") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= x"06B"; --zero
            end if;
        elsif(state = x"06B") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"06C"; --space
            end if;
        elsif(state = x"06C") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"06D"; --zero
            end if;
        elsif(state = x"06D") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"06E"; --zero
            end if;
        elsif(state = x"06E") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"06F"; --space
            end if;
        elsif(state = x"06F") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"070"; --zero
            end if;
        elsif(state = x"070") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"071"; --zero
            end if;
        elsif(state = x"071") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"072"; --]
            end if;
        elsif(state = x"072") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= x"073"; --BS
            end if;
        elsif(state = x"073") then
            if(tx_done = '1' AND tx_active = '0' AND row_bs_count < 8) then
                row_bs_count <= row_bs_count + 1; --
            elsif(tx_done = '1' AND tx_active = '0') then
                state <= x"074"; --Receive
                row_bs_count <= 0;
            end if;
        elsif(state = x"074") then
            if(received = '1' AND valid(data_rx) ) then
                state <= x"075"; --Send received data
                    if(row2_state(5) = '0') then
                        row2_bcd(23 downto 20) <= ascii_t_bcd(data_rx); --Store input
                        row2_state(5) <= '1';
                    else
                        row2_bcd(19 downto 16) <= ascii_t_bcd(data_rx); --Store input
                        row2_state(4) <= '1';
                    end if;
            end if;
        elsif(state = x"075") then
            if(tx_done = '1' AND tx_active = '0') then
                if(row2_state(4) = '1') then
                    state <= X"076"; --Space
                else
                    state <= X"074"; --Receive
                end if;
            end if;
        elsif(state = x"076") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"077"; --Receive
            end if;
        elsif(state = x"077") then
            if(received = '1' AND valid(data_rx) ) then
                state <= x"078"; --Send received data
                if(row2_state(3) = '0') then
                    row2_bcd(15 downto 12) <= ascii_t_bcd(data_rx); --Store input
                    row2_state(3) <= '1';
                else
                    row2_bcd(11 downto 8) <= ascii_t_bcd(data_rx); --Store input
                    row2_state(2) <= '1';
                end if;
            end if;
        elsif(state = x"078") then
            if(tx_done = '1' AND tx_active = '0') then
                if(row2_state(2) = '1') then
                    state <= X"079"; --Space
                else
                    state <= X"077"; --Receive
                end if;
            end if;
        elsif(state = x"079") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"07A"; --Receive
            end if;
        elsif(state = x"07A") then
            if(received = '1' AND valid(data_rx) ) then
                state <= x"07B"; --Send received data
                if(row2_state(1) = '0') then
                    row2_bcd(7 downto 4) <= ascii_t_bcd(data_rx); --Store input
                    row2_state(1) <= '1';
                else
                    row2_bcd(3 downto 0) <= ascii_t_bcd(data_rx); --Store input
                    row2_state(0) <= '1';
                end if;
            end if;
        elsif(state = x"07B") then
            if(tx_done = '1' AND tx_active = '0') then
                if(row2_state(0) = '1') then
                    state <= X"07C"; --LF
                else
                    state <= X"07A"; --Receive
                end if;
            end if;
        elsif(state = x"07C") then
            r1c1 <= bcd_t_binary(row1_bcd(23 downto 16));
            r1c2 <= bcd_t_binary(row1_bcd(15 downto 8));
            r1c3 <= bcd_t_binary(row1_bcd(7 downto 0));
            r2c1 <= bcd_t_binary(row2_bcd(23 downto 16));
            r2c2 <= bcd_t_binary(row2_bcd(15 downto 8));
            r2c3 <= bcd_t_binary(row2_bcd(7 downto 0));
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"07D"; --CR
            end if;
        elsif(state = x"07D") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"07E"; --Check for valid input
            end if;
        elsif(state = x"07E") then
            if((r1c1 > max_input OR r1c2 > max_input OR r1c3 > max_input OR r2c1 > max_input OR r2c2 > max_input OR r2c3 > max_input) AND tx_done = '1' AND tx_active = '0') then
                state <= X"07F"; --I
            elsif(tx_done = '1' AND tx_active = '0') then
                state <= X"08F";
            end if;
        elsif(state = x"07F") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"080"; --n
            end if;
        elsif(state = x"080") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"081"; --v
            end if;
        elsif(state = x"081") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"082"; --a
            end if;
        elsif(state = X"082") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"083"; --l
            end if;
        elsif(state = x"083") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"084"; --i
            end if;
        elsif(state = x"084") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"085"; --d
            end if;
        elsif(state = x"085") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"086"; --space
            end if;
        elsif(state = x"086") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"087"; --I
            end if;
        elsif(state = x"087") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"088"; --n
            end if;
        elsif(state = x"088") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"089"; --p
            end if;
        elsif(state = x"089") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"08A"; --u
            end if;
        elsif(state = x"08A") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= x"08B"; --t
            end if;
        elsif(state = x"08B") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"08C"; --LF
            end if;
        elsif(state = x"08C") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"08D"; --LF
            end if;
        elsif(state = x"08D") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"08E"; --CR
            end if;
        elsif(state = x"08E") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"000"; --Reset State
            end if;
        elsif(state = x"08F") then
            sol_x_int <= to_integer(unsigned(sol_x));
            sol_y_int <= to_integer(unsigned(sol_y));
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"097"; --S
            end if;
        elsif(state = x"090") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"091"; --one
            end if;
        elsif(state = x"091") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"092"; --space
            end if;
        elsif(state = x"092") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= x"093"; --zero
            end if;
        elsif(state = x"093") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"094"; --space
            end if;
        elsif(state = x"094") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"095"; --tens
            end if;
        elsif(state = x"095") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= x"096"; --ones
            end if;
        elsif(state = x"096") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"0A3"; --]
            end if;
        elsif(state = x"097") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= x"098"; --o
            end if;
        elsif(state = x"098") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"099"; --l
            end if;
        elsif(state = x"099") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"09A"; --u
            end if;
        elsif(state = x"09A") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= x"09B"; --t
            end if;
        elsif(state = x"09B") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"09C"; --i
            end if;
        elsif(state = x"09C") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"09D"; --o
            end if;
        elsif(state = x"09D") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= x"09E"; --n
            end if;
        elsif(state = x"09E") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"09F"; --colon
            end if;
        elsif(state = x"09F") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= x"0A0"; --FL
            end if;
        elsif(state = x"0A0") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= x"0A1"; --CR
            end if;
        elsif(state = x"0A1") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"0A2"; --space
            end if;
        elsif(state = x"0A2") then
            if(tx_done = '1' AND tx_active = '0' AND tab_space < 3) then
                tab_space <= tab_space + 1;
            elsif(tx_done = '1' AND tx_active = '0') then
                state <= X"090"; --[
                tab_space <= 0;
            end if;
        elsif(state = x"0A3") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"0A4"; --LF
            end if;
        elsif(state = x"0A4") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"0A5"; --CR
            end if;
        elsif(state = x"0A5") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= x"0A6"; --space
            end if;
        elsif(state = x"0A6") then
            if(tx_done = '1' AND tx_active = '0' AND tab_space < 3) then
                tab_space <= tab_space + 1;
            elsif(tx_done = '1' AND tx_active = '0') then
                state <= X"0A7"; --[
            end if;
        elsif(state = x"0A7") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"0A8"; --zero 
            end if;
        elsif(state = x"0A8") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= x"0A9"; --space
            end if;
        elsif(state = x"0A9") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"0AA"; --one
            end if;
        elsif(state = x"0AA") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= x"0AB"; --space
            end if;
        elsif(state = x"0AB") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= x"0AC"; --tens
            end if;
        elsif(state = x"0AC") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"0AD"; --ones
            end if;
        elsif(state = x"0AD") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= x"0AE"; --]
            end if;
        elsif(state = x"0AE") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"0AF"; --LF
            end if;
        elsif(state = x"0AF") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= X"0B0"; --CR
            end if;
        elsif(state = x"0B0") then
            if(tx_done = '1' AND tx_active = '0') then
                state <= x"000"; --RESET
            end if;
            
        end if;
    end if;
end process;

with state select
    send <= '0' when x"000",
            '0' when x"052",
            '0' when x"055",
            '0' when x"058",
            '0' when x"074",
            '0' when x"077",
            '0' when x"07A",
            '1' when others;

--0000101000001101 /r/n
with state select
    data_tx <= x"00" when x"000",
                     C_W when x"001",
                     e when x"002",
                     l when x"003",
                     c when x"004",
                     o when x"005",
                     m when x"006",
                     e when x"007",
                     space when x"008",
                     t when x"009",
                     o when x"00A",
                     space when x"00B",
                     t when x"00C",
                     h when x"00D",
                     e when x"00E",
                     space when x"00F",
                     C_B when x"010",
                     a when x"011",
                     s when x"012",
                     y when x"013",
                     s when x"014",
                     three when x"015",
                     space when x"016",
                     C_M when x"017",
                     a when x"018",
                     t when x"019",
                     r when x"01A",
                     i when x"01B",
                     x when x"01C",
                     space when x"01D",
                     C_S when x"01E",
                     o when x"01F",
                     l when x"020",
                     v when x"021",
                     e when x"022",
                     r when x"023",
                     LF when x"024",
                     CR when x"025",
                     C_E when x"026",
                     n when x"027",
                     t when x"028",
                     e when x"029",
                     r when x"02A",
                     space when x"02B",
                     a when x"02C",
                     space when x"02D",
                     two when x"02E",
                     x when x"02F",
                     three when x"030",
                     space when x"031",
                     m when x"032",
                     a when x"033",
                     t when x"034",
                     r when x"035",
                     i when x"036",
                     x when x"037",
                     colon when x"038",
                     LF when x"039",
                     CR when x"03A",
                     space when x"03B",
                     C_F when x"03C",
                     i when x"03D",
                     r when  x"03E",
                     s when x"03F",
                     t when x"040",
                     space when x"041",
                     r when x"042",
                     o when x"043",
                     w when x"044",
                     colon when x"045",
                     space when x"046",
                     l_s_b when x"047",
                     zero when x"048",
                     zero when x"049",
                     space when x"04A",
                     zero when x"04B",
                     zero when x"04C",
                     space when x"04D",
                     zero when x"04E",
                     zero when x"04F",
                     r_s_b when x"050",
                     BS when x"051",
                     data_rx when x"053",
                     space when x"054",
                     data_rx when X"056",
                     space when X"057",
                     data_rx when X"059",
                     LF when X"05A",
                     CR when X"05B",
                     space when x"05C",
                     C_S when x"05D",
                     e when x"05E",
                     c when x"05F",
                     o when x"060",
                     n when x"061",
                     d when x"062",
                     space when x"063",
                     r when x"064",
                     o when x"065",
                     w when x"066",
                     colon when x"067",
                     space when x"068",
                     l_s_b when x"069",
                     zero when x"06A",
                     zero when x"06B",
                     space when x"06C",
                     zero when x"06D",
                     zero when x"06E",
                     space when x"06F",
                     zero when x"070",
                     zero when x"071",
                     r_s_b when x"072",
                     BS when x"073",
                     data_rx when x"075",
                     space when x"076",
                     data_rx when x"078",
                     space when x"079",
                     data_rx when x"07B",
                     LF when x"07C",
                     CR when x"07D",
                     C_I when x"07F",
                     n when x"080",
                     v when x"081",
                     a when x"082",
                     l when x"083",
                     i when x"084",
                     d when x"085",
                     space when x"086",
                     C_I when x"087",
                     n when x"088",
                     p when x"089",
                     u when x"08A",
                     t when x"08B",
                     LF when x"08C",
                     LF when x"08D",
                     CR when x"08E",
                     l_s_b when x"090",
                     one when x"091",
                     space when x"092",
                     zero when x"093",
                     space when x"094",
                     tens(sol_x_int) when x"095",
                     ones(sol_x_int) when x"096",
                     r_s_b when x"0A3",
                     C_S when x"097",
                     o  when x"098",
                     l when x"099",
                     u when  x"09A",
                     t when x"09B",
                     i when x"09C",
                     o when x"09D",
                     n when x"09E",
                     colon when x"09F",
                     LF when x"0A0",
                     CR when x"0A1",
                     space when x"0A2",
                     LF when x"0A4",
                     CR when x"0A5",
                     space when x"0A6",
                     l_s_b when x"0A7",
                     zero when x"0A8",
                     space when x"0A9",
                     one when x"0AA",
                     space when x"0AB",
                     tens(sol_y_int) when x"0AC",
                     ones(sol_y_int) when x"0AD",
                     r_s_b when x"0AE",
                     LF when x"0AF",
                     CR when x"0B0",
                     x"00" when others;


----------------------------------------------------------------------------------
 -- Map Debug PMOD Ports
----------------------------------------------------------------------------------
--TXD_DEBUG <= TXD_SIG;
--RXD_DEBUG <= RXD;
--module_start_debug <= module_start;
--state_debug <= state(3 downto 0);
--tx_done_debug <= tx_done;
----------------------------------------------------------------------------------
 -- Map LED Debug Ports
----------------------------------------------------------------------------------
led(9) <= send;
led(10) <= tx_done;
led(11) <= received;
led(15 downto 12) <= state(3 downto 0);
----------------------------------------------------------------------------------
 -- Other
----------------------------------------------------------------------------------
TXD <= TXD_SIG;


end Behavioral;
